c eVSCF - Variant of VSCF. A particular combination of excited product
c functions must be self-consistent, i.e., the averaged potential
c is calculated as <a_1..a_(i-1),a_(i+1)..a_n|V|...>, whereas the
c original version was <0..i-1,i+1..0|V|...>.
c
c See ../docvib/evscf-pt.latex file for the theory.
c
c
      subroutine calc_evscf_energies(nnmodes,vscf_nbase,func_exc, 
     1qff2,qff3,qff4, ept0,ept1)
      include 'common.h'
c In:
      integer nnmodes,vscf_nbase,func_exc(nnmodes)
      real*8 qff2(nnmodes), qff3(nnmodes,nnmodes,nnmodes)
      real*8 qff4(nnmodes,nnmodes,nnmodes)
      
c Out:      
      real*8 ept0,ept1
      
      ! The parameters of vscf_funcs are: (excitations,vscf_base_funcs_coefficients,modes)
      real*8 vscf_funcs(vscf_nbase,vscf_nbase,nnmodes)

      ! The one-mode VSCF energies of excited states
      real*8  vscf_energies(vscf_nbase,nnmodes)

      ! The first index goes over the base of LHOs, the second one over the modes
      real*8 vscf_func(vscf_nbase,nnmodes)




c     maxediff - convergence criteria for the VSCF iterations
      real*8 maxediff, ept0_old

c     maxiter  - maximal count of VSCF iterations
      integer maxiter, iiter
      logical vscf_done

      
      
c 1. Find the self-consistent excited function and the PT0 energy
c
      maxediff = GVIB_VSCF_CLIMIT
      maxiter  = GVIB_VSCF_MAXITER


      call init_evscf_func(vscf_nbase,nnmodes,func_exc,vscf_func)

      if (verbose) then
	    write (6,*)
     1 'Starting eVSCF computation in base of ',vscf_nbase,' LHOs...'
	    write (log_fd,*) '   for the product function ', func_exc   
      end if
      
      ept0_old=0
      iiter=1
      vscf_done=.false.
      do while ( .not.vscf_done )
      
            call evscf_iteration(vscf_nbase,nnmodes,qff2,
     1    qff3,qff4,func_exc, vscf_func, vscf_funcs,vscf_energies,ept0)

            if (verbose) write(log_fd,*)iiter,'..  eVSCF E=',
     1       ept0*V2sqrt_to_cminv,' [1/cm]; dE=',
     2       (ept0_old-ept0)*V2sqrt_to_cminv,' [1/cm]'

            if (iiter.gt.2) then
	      if ( maxiter.gt.0 .and. iiter.gt.maxiter ) 
     1        vscf_done = .true.
	      if ( V2sqrt_to_cminv*abs(ept0_old-ept0).lt.maxediff ) 
     1        vscf_done = .true.
	      end if
            
            ept0_old = ept0
            iiter    = iiter+1
      end do

      if (verbose) then
	    write(log_fd,*)'  -> converged after ',iiter,' iterations.'
      else
	    write(log_fd,*)'eVSCF: ',vscf_nbase,' LHOs, ',iiter-1,
     &		  ' iterations, error ',
     1           V2sqrt_to_cminv*abs(ept0_old-ept0),' [1/cm]'
      end if




c 2. Calculate the PT1 and PT2 contribution to the energy
c
c      call calc_evscfpt12_energy(nnmodes,vscf_nbase,qff2,qff3,qff4,func_exc,vscf_funcs,vscf_energies,ept1,ept2)
      call calc_vscfpt1_energy(nnmodes,vscf_nbase,vscf_func,qff2,
     1qff3,qff4,ept1)

      end 
