      SUBROUTINE MITINS(N,M,NE,ELMAX,D,IERR,MV,J1,s1,MVTOT,IEMAX,
     1nproc,clim,MAXIT,HW,OLIMIT,GRADCONV,AMACH,acoef,lcoef,dele)
C     variant of mitinx for sparse matrices and sparse eigenvectors
c     (most |Ci| < clim)
c     g95 friendly
C
C     by Petr Bour, Prague 1994
C
C     This subroutine finds few of lowest eigenvectors for a
C     very large matrix, written on the disk (s1//A2.SCR)
C     Diagonal terms are already divided by two in s1//.SCR.
C     M is the dimension of the subspace where direct diagonalization is done
C     NE is number of eigenvectors to be found
C     MV is the number of eigenvectors found
C     MVTOT is the number of eigenvectors found in all symmetry blocks
c     N is matrix dimension
C
C     For details look at
C     Alexander V. Mitin, J.Comp.Chem., Vol. 15, No.7, (1994), 747-751
C     (Warning: The paper is full of missprints; the recommended procedure
C      is modified here.)
C
      IMPLICIT none
      character*1 s1
      integer*4 N,M,NE,IERR,MV,J1,MVTOT,IEMAX,it0,mclock,MSAVE,nproc,
     1i,ndone,IERM,iel,ii,IN,IEE,IE,MSTART,J,ITER,MAXIT,K,NK2,
     1NK1,NIN,NM1,JJ,MNEW,ipr,MMAX,
     1OMP_GET_MAX_THREADS,OMP_GET_NUM_PROCS,ielt,
     1OMP_GET_THREAD_NUM
      real*8 ELAST,GRADOLD,ANORM,DIAG,clim,adf,EL,AF,SN,dell,s1r,
     1CONV,FF,OLIMIT,GRADCONV,ALCONV,AMACH,ELMAX,D(*),SUM,acoef,
     1abbm,abbm1,abbm2,ecoef,dele
      logical l12,HW,lcoef
      real*8,allocatable:: abb(:),a33(:),cev(:),c1(:),E(:),YNEW(:),X(:),
     1AXV(:),F(:,:),Z(:,:),BUF(:),sar(:),SNS(:),tem(:)
c     abb(nbuff)
      integer*4,allocatable::
     1 jbb(:),nstart(:),nend(:),nn(:),j33(:),mm(:),nev(:),
     1iev(:),i1(:),cfrom(:),item(:),nstartt(:),nendt(:)
c     jbb(nbuff)
c
      call w36(' ')
      call w36('Mitins')
      call w36('^^^^^^')
      AF=1.0d0/DSQRT(DBLE(N))
      it0=mclock()
      ELAST=-1.0d9
      GRADOLD=-1.0d9
      MSAVE=M
      IERR=0
      ANORM=0.0d0
      DIAG=0.0d0
      MMAX=M
c     matrix space:
      allocate(j33(N),a33(N),nn(N),mm(N),c1(N),i1(N),cfrom(N),
     1nev(N),YNEW(N),SNS(N))
c     iterative space:
      allocate(E(M),Z(M,M),X(M*N),AXV(M*N),F(M,M),BUF(M))
      do 8 I=1,M
      E(I)=0.0d0
      do 8 J=1,M
8     Z(I,J)=0.0d0
      allocate(sar(nproc))

c     set up parallel parameters:
      if(nproc.gt.omp_get_num_procs())then
       write(6,*)nproc,omp_get_num_procs()
       call report('not enough processors')
      endif
      if(nproc.gt.omp_get_max_threads())
     1write(6,*)'Number of threads will increase'
      call omp_set_num_threads(nproc)
      call omp_set_dynamic(0)
      call omp_set_nested(0)
      allocate(nstart(nproc),nend(nproc),nstartt(nproc),nendt(nproc))
c     precalculate parallel distributions for "non-parralezable" loops:
      call ppd(nproc,nproc,nstart,nend,N)
      do i=1,nproc
       write(6,*)i,nstart(i),nend(i)
      enddo
c
      ndone=0
      OPEN(45,FILE='CFIs.SCR',FORM='UNFORMATTED')
c     previous block:
      do 2002 I=1,J1-1
      ndone=ndone+1
2002  READ(45)
c     current block:
      IERM=0
2001  READ(45,END=2004)
      ndone=ndone+1
      IERM=IERM+1
      goto 2001
2004  write(3,3400)ndone,IERM
      write(6,3400)ndone,IERM
3400  format(i5,' eigenvectors found in CFIs.SCR,',i5,' in this block')

      inquire(FILE=s1//'A2.SCR',exist=l12)
      if(l12)then
       OPEN(2,FILE=s1//'A2.SCR',FORM='UNFORMATTED')
       adf=1.0d0
      else
       OPEN(2,FILE=s1//'A.SCR',FORM='UNFORMATTED')
       adf=2.0d0
      endif
      read(2)
      iel=0
      DO 222 I=1,N
      READ(2)nn(I),(j33(ii),ii=1,nn(I)),(a33(ii),ii=1,nn(I))
      DO 222 ii=1,nn(I)
      iel=iel+1
      if(j33(ii).eq.I)DIAG=DIAG+DABS(a33(ii))*adf
222   ANORM=ANORM+ABS(a33(ii))
      close(2)

      write(6,4001)iel
      write(3,4001)iel
4001  format(i10,' elements')
      allocate(jbb(iel),abb(iel))
      ielt=iel
      call loadh(N,j33,a33,jbb,abb,nn,mm,s1)
      write(6,4901)IERM,J1
      write(3,4901)IERM,J1
4901  format(I8,' eigenvalues loaded now, J1 = ',I8)
      REWIND 45
      DO 1005 IN=1,J1-1
1005  READ(45)
c     eigenvalue_l = sum(j=1..nev(l))cev(i(j))xi(i(j))
      iel=0
      DO 1007 IN=1,IERM
c     read in temporary strings:
      READ(45)nev(IN),(c1(I),I=1,nev(IN)),(i1(I),I=1,nev(IN))
      cfrom(IN)=iel+1
      iel=iel+nev(IN)
      if(IN.eq.1)then
c      allocate space:
       allocate(cev(cfrom(IN)+nev(IN)-1),iev(cfrom(IN)+nev(IN)-1))
c      just to avoid compile messages:
       iev(1)=0
       cev(1)=0.0d0
      else
c      change array dimensions:
       allocate(tem(cfrom(IN-1)+nev(IN-1)-1),
     1 item(cfrom(IN-1)+nev(IN-1)-1))
       call eqvr(tem,cev,cfrom(IN-1)+nev(IN-1)-1)
       call eqvi(item,iev,cfrom(IN-1)+nev(IN-1)-1)
       deallocate(cev,iev)
       allocate(cev(cfrom(IN)+nev(IN)-1),iev(cfrom(IN)+nev(IN)-1))
       call eqvr(cev,tem,cfrom(IN)-1)
       call eqvi(iev,item,cfrom(IN)-1)
       deallocate(tem,item)
      endif
c     fill the string:
      do 1007 I=1,nev(IN)
      iev(cfrom(IN)+I-1)=i1(I)
1007  cev(cfrom(IN)+I-1)=c1(I)

      IF(N.LT.M)THEN
       WRITE(6,*)' Sub-space dimension must be =< N !',N,M
       WRITE(3,*)' Sub-space dimension must be =< N !',N,M
       IERR=2
       GOTO 6666
      ENDIF
      ANORM=2.0d0*ANORM/(DBLE(N*(N-1)))
      DIAG=DIAG/DBLE(N)*2.0d0
      ALCONV=GRADCONV*MAX(DIAG,ANORM)*DBLE(N)
      WRITE(3,3003)ANORM,DIAG, ALCONV,OLIMIT,MAXIT,N,M,IERM+1,J1,s1,
     1MVTOT,ELMAX,IEMAX,l12,nproc,lcoef,acoef,dele,clim
      WRITE(6,3003)ANORM,DIAG, ALCONV,OLIMIT,MAXIT,N,M,IERM+1,J1,s1,
     1MVTOT,ELMAX,IEMAX,l12,nproc,lcoef,acoef,dele,clim
3003  FORMAT(' Average magnitude of matrix el. :',F20.10,/,
     1       ' Average diagonal element        :',F20.10,/,
     2       ' Convergence criterium           :',F20.10,/,
     3       ' Cut-off norm for subspace       :',F20.10,/,
     4       ' Maximum number of iterations    :',I20,/,
     5       ' Dimension of the matrix space   :',I20,/,
     6       ' Dimension of the iterative space:',I20,/,
     7       ' Restart eigenvalue              :',I20,/,
     8       ' Total eigenvalue                :',I20,/,
     8       ' Symmetry block                  :',19x,a1,/,
     8       ' Total eigenvalues done          :',I20,/,
     9       ' Maximal eigenvalue              :',F20.2,/,
     9       ' Maximal number of eigenvalues   :',I20,/,
     9       ' l12                             :',L20,/,
     9       ' Number of processors            :',I20,/,
     9       ' Large coefficient preference    :',L20,/,
     9       ' Large coefficient coupling      :',F20.4,/,
     9       ' Energy change threshold         :',F20.10,/,
     9       ' Coefficient cutoff limit        :',F20.10,/,/)
c
      OPEN(4,FILE=s1//'E.SCR',FORM='UNFORMATTED')
      READ(4,END=444)(D(IEE),IEE=1,IERM)
444   CLOSE(4)
      WRITE(6,6008)IERM+1
      WRITE(3,6008)IERM+1
6008  FORMAT(' Calculation started from eigenvalue number ',I4)
c
C     Starting dimension of the subspace:
      MSTART=M
C
      DO 5555 IE=IERM+1,MIN(N,NE)
C     Estimate starting eigenvalue:
      EL=0.0d0
      IF(IE.GT.1)EL=D(IE-1)
C     Estimate starting eigenvector as unit vector:
      DO 29 I=1,N
29    X(I)=AF
c     For higher vectors, take that from previous iteration
      if(IE.GT.IERM+1)then
       EL=E(2)
C$OMP  PARALLEL DO PRIVATE(I,J)  DEFAULT(SHARED)
       DO 131 I=1,N
       YNEW(I)=0.0d0
       DO 131 J=1,M
131    YNEW(I)=YNEW(I)+Z(J,2)*X(I+N*(J-1))
C$OMP  PARALLEL DO PRIVATE(I)  DEFAULT(SHARED)
       DO 241 I=1,N
241    X(I)=YNEW(I)
      endif

      ITER=0
      DELL=0.0d0
      M=MSTART
C
C     Start of the iterative cycle: ..............................
C
9999  ITER=ITER+1
      IF(ITER.GT.MAXIT)THEN
       WRITE(3,*)' Maximum number of iterations exceeded'
       WRITE(6,*)' Maximum number of iterations exceeded'
       IERR=8888
       GOTO 8888
      ENDIF
      IF(M+IE-1.GT.N)THEN
       WRITE(3,*)' Subspace-dimension has to be decreased'
       M=N-IE+1
       IF(M.LT.1)THEN
        WRITE(3,*)'  No subspace for the eigenvector left !'
        WRITE(3,*)' Try to increase the subspace-dimension.'
        GOTO 6666
       ENDIF
      ENDIF

C     Orthonormalize to previous eigenvectors Y1..YIE-1 in this block:
      DO 711 IN=1,IE-1
      call ppd(nproc,nproc,nstartt,nendt,nev(IN))
C$OMP PARALLEL PRIVATE(I,SN,ipr)  DEFAULT(SHARED)
      SN=0.0d0
      ipr=OMP_GET_THREAD_NUM()+1
      DO 611 I=cfrom(IN)+nstartt(ipr)-1,cfrom(IN)+nendt(ipr)-1
611   SN=SN+cev(I)*X(iev(I))
      sar(ipr)=SN
C$OMP END PARALLEL
      SN=0.0d0
      DO 6111 I=1,nproc
6111  SN=SN+sar(I)
C$OMP PARALLEL DO PRIVATE(I)  DEFAULT(SHARED)
      DO 7111 I=cfrom(IN),cfrom(IN)+nev(IN)-1
7111  X(iev(I))=X(iev(I))-SN*cev(I)
711   CONTINUE

      call normvc(X,1,N,nstartt,nendt,nproc,sar)

C     Form gradient vectors Xk,k=2..M, new basis: {X1,..,Xk}
C     Xk+1=AXk-ELXk  (Xk,K>1 generated by X1)
C$OMP PARALLEL DO PRIVATE(I)  DEFAULT(SHARED)
      DO 209 I=1,N*M
209   AXV(I)=0.0d0

      DO 3 K=2,M
      NK2=N*(K-2)
      if(l12)then
       if(lcoef)then
C$OMP   PARALLEL DO PRIVATE(I,ii,J,s1r,abbm)  DEFAULT(SHARED)
        DO 203 I=1,N
        s1r=0.0d0
        DO 20311 ii=1,nn(I)
        J=mm(I)+ii
        if(jbb(J).eq.I)then
         abbm=abb(J)-acoef*X(jbb(J)+NK2)**2
        else
         abbm=abb(J)
        endif
20311   s1r=s1r+abbm*X(jbb(J)+NK2)
c       AXV(K,I) = A(I,J) * X(K-1,J)
203     AXV(I+NK2)=s1r
       else
C$OMP   PARALLEL DO PRIVATE(I,ii,J,s1r)  DEFAULT(SHARED)
        DO 2032 I=1,N
        s1r=0.0d0
        DO 20321 ii=1,nn(I)
        J=mm(I)+ii
20321   s1r=s1r+abb(J)*X(jbb(J)+NK2)
2032    AXV(I+NK2)=s1r
       endif
       
      else
       DO 204 I=1,N
       DO 204 ii=1,nn(I)
       J=jbb(mm(I)+ii)
       if(lcoef.and.J.eq.I)then
        abbm1=abb(mm(I)+ii)-acoef*X(I+NK2)**2
        abbm2=abb(mm(I)+ii)-acoef*X(J+NK2)**2
       else
        abbm1=abb(mm(i)+ii)
        abbm2=abb(mm(i)+ii)
       endif
       AXV(J+N*(K-2))=AXV(J+N*(K-2))+abbm1*X(I+NK2)
204    AXV(I+N*(K-2))=AXV(I+N*(K-2))+abbm2*X(J+NK2)
      endif
C     Gradient vector K:    Xk+1=AXk-ELXk :
      NK1=N*(K-1)
      NK2=N*(K-2)
C$OMP PARALLEL DO PRIVATE(I)  DEFAULT(SHARED)
      DO 2031 I=1,N
2031  X(I+NK1)=AXV(I+NK2)-EL*X(I+NK2)
C     Orthogonalize vector XK to previous gradient vectors:
C$OMP PARALLEL DO PRIVATE(I,IN,SN,NIN)  DEFAULT(SHARED)
      DO 74 IN=1,K-1
      SN=0.0d0
      NIN=N*(IN-1)
      DO 741 I=1,N
741   SN=SN+X(I+NIN)*X(I+NK1)
74    SNS(IN)=SN

C$OMP PARALLEL DO PRIVATE(I,IN)  DEFAULT(SHARED)
      DO 7 I=1,N
      DO 7 IN=1,K-1
7     X(I+NK1)=X(I+NK1)-SNS(IN)*X(I+N*(IN-1))

C     Orthogonalize vector XK to previous eigenvectors Xin, in=1..IE-1
C$OMP PARALLEL DO PRIVATE(I,IN,SN,NIN)  DEFAULT(SHARED)
      DO 1008 IN=1,IE-1
      SN=0.0d0
      DO 1009 I=cfrom(IN),cfrom(IN)+nev(IN)-1
1009  SN=SN+cev(I)*X(iev(I)+NK1)
1008  SNS(IN)=SN
C$OMP PARALLEL DO PRIVATE(I,IN)  DEFAULT(SHARED)
      DO 722 IN=1,IE-1
      DO 722 I=cfrom(IN),cfrom(IN)+nev(IN)-1
722   X(iev(I)+NK1)=X(iev(I)+NK1)-SNS(IN)*cev(I)
C     Normalize vector K:

C$OMP PARALLEL DEFAULT(SHARED) PRIVATE(ipr,j,s1r)
      ipr=OMP_GET_THREAD_NUM()+1
      s1r=0.0d0
      do 110 j=nstart(ipr),nend(ipr)
110   s1r=s1r+X(j+NK1)**2
      sar(ipr)=s1r
C$OMP END PARALLEL
      ANORM=0.0d0
      DO 10 I=1,nproc
10    ANORM=ANORM+sar(I)

C     (Check on convergence on gradient for the last:)
      IF(K.EQ.2)then
       CONV=ANORM
       if(DABS(CONV).LT.ALCONV)GOTO 8888
      endif
      
      ANORM=1.0d0/DSQRT(ANORM)
C$OMP PARALLEL DO PRIVATE(I)  DEFAULT(SHARED)
      DO 5 I=1,N
5     X(I+NK1)=X(I+NK1)*ANORM
3     CONTINUE
c     end of K -loop of gradient vectors

      NM1=N*(M-1)
      if(l12)then
       if(lcoef)then
C$OMP   PARALLEL DO PRIVATE(I,ii,j,s1r,abbm)  DEFAULT(SHARED)
        DO 207 I=1,N
        s1r=0.0d0
        DO 2071 ii=1,nn(I)
        J=mm(I)+ii
        if(jbb(J).eq.I)then
         abbm=abb(J)-acoef*X(jbb(J)+NM1)**2
        else
         abbm=abb(J)
        endif
2071    s1r=s1r+abbm*X(jbb(J)+NM1)
207     AXV(I+NM1)=s1r
       else
C$OMP   PARALLEL DO PRIVATE(I,ii,J,s1r)  DEFAULT(SHARED)
        DO 2072 I=1,N
        s1r=0.0d0
        DO 20721 ii=1,nn(I)
        J=mm(I)+ii
20721   s1r=s1r+abb(J)*X(jbb(J)+NM1)
2072    AXV(I+NM1)=s1r
       endif
      else
       DO 208 I=1,N
       DO 208 ii=1,nn(I)
       J=jbb(mm(I)+ii)
       if(lcoef.and.J.eq.I)then
        abbm1=abb(mm(I)+ii)-acoef*X(I+NM1)**2
        abbm2=abb(mm(I)+ii)-acoef*X(J+NM1)**2
       else
        abbm1=abb(mm(i)+ii)
        abbm2=abb(mm(i)+ii)
       endif
       AXV(J+N*(M-1))=AXV(J+N*(M-1))+abbm1*X(I+NM1)
208    AXV(I+N*(M-1))=AXV(I+N*(M-1))+abbm2*X(J+NM1)
      endif
C     Now looking for new eigenvector in the subspace of {Xk}
C     Form the sub-space matrix F=X.A.X:
C$OMP PARALLEL DO PRIVATE(ii,jj,i)  DEFAULT(SHARED)
      DO 21 II=1,M
      DO 21 JJ=1,II
      F(II,JJ)=0.0d0
      DO 21 I=1,N
21    F(II,JJ)=F(II,JJ)+AXV(I+N*(II-1))*X(I+N*(JJ-1))
C     Diagonalize the sub-space directly:
      CALL TRED12(MMAX,M,F,Z,E,2,IERR,BUF)
C     Take the lowest eigenvalue:
      EL=E(1)
      if(HW)WRITE(6,6007)ITER,EL,CONV,M
6007  FORMAT(' It=',I4,' Lambda=',F20.10,' Grad=',F20.10,' M=',I3)
      DELL=ABS(ELAST-EL)
      if(DELL.le.dele)then
       write(6,6004)
6004   format(' Energy change smaller than the limit')
       goto 8888
      endif
      ELAST=EL
C     New approximation to the eigenvector:
C$OMP PARALLEL DO PRIVATE(j,i,SN)  DEFAULT(SHARED)
      DO 24 I=1,N
      SN=0.0d0
      DO 13 J=1,M
13    SN=SN+Z(J,1)*X(I+N*(J-1))
      YNEW(I)=SN
24    X(I)=SN
C     Check occasional fault of direct diagonalization:
C$OMP PARALLEL PRIVATE(i,ANORM,ipr)  DEFAULT(SHARED)
      ipr=OMP_GET_THREAD_NUM()+1
      sar(ipr)=0.0d0
      DO 224 I=nstart(ipr),nend(ipr)
224   sar(ipr)=sar(ipr)+ X(I)**2
C$OMP END PARALLEL
      ANORM=0.0d0
      DO 2241 I=1,nproc
2241  ANORM=ANORM+sar(I)
      IF(ANORM.LT.AMACH)THEN
C$OMP  PARALLEL DO PRIVATE(i)  DEFAULT(SHARED)
       DO 225 I=1,N
       X(I)=AF
225    YNEW(I)=AF
      ENDIF
      IF(M.EQ.1)THEN
       WRITE(3,*)'Last eigenvector left'
       GOTO 8888
      ENDIF
c
C     Check if the sub-space is not redundant:
      MNEW=M
      DO 50 J=M,4,-1
      FF=Z(J,1)*Z(J,1)
      IF(FF.GT.OLIMIT)GOTO 4444
50    MNEW=J-1
4444  M=MAX(MNEW,3)
C
C     Check if the sub-space should be increased, up to original M:
      FF=Z(M,1)*Z(M,1)
      IF(FF.GT.OLIMIT)M=MIN(M+1,MMAX)
      IF(ITER.GT.1)THEN
       IF(CONV.GE.GRADOLD)M=MIN(M+1,MMAX)
      ENDIF
C
      GRADOLD=CONV
      GOTO 9999
8888  WRITE(3,6000)IE,ITER-1,CONV,EL,ALCONV,DELL,MNEW
     1,dble(mclock()-it0)/100.0d0
      WRITE(6,6000)IE,ITER-1,CONV,EL,ALCONV,DELL,MNEW
     1,dble(mclock()-it0)/100.0d0
6000  Format(
     1' *************** The ',I3,'. eigenvalue ******************',/,
     2' Iter',I6,' Gradient   : ',F20.10,' Lambda  : ',F20.10,/,
     3'     ',6X,' Limit grad.: ',F20.10,' Del Lam.: ',F20.10,/,
     4' Final sub-space dim.  : ',   I20,' t (sec) : ',f20.2)
      D(IE)=EL
c     select large coefficients only:
      nev(IE)=0
      do 1 J=1,N
      if(dabs(YNEW(J)).gt.clim)then
       nev(IE)=nev(IE)+1
       c1(nev(IE))=YNEW(J)
       i1(nev(IE))=J
      else
       YNEW(J)=0.0d0
      endif
1     continue
      write(3,6001)nev(IE)
      write(6,6001)nev(IE)
6001  format(I9,' coefficients larger than the limit') 
c     allocate space for it:
      if(IE.eq.1)then
       cfrom(IE)=1
       allocate(cev(cfrom(IE)+nev(IE)-1),iev(cfrom(IE)+nev(IE)-1))
      else
       cfrom(IE)=cfrom(IE-1)+nev(IE-1)
       allocate(tem(cfrom(IE-1)+nev(IE-1)-1),
     1 item(cfrom(IE-1)+nev(IE-1)-1))
       call eqvr(tem,cev,cfrom(IE-1)+nev(IE-1)-1)
       call eqvi(item,iev,cfrom(IE-1)+nev(IE-1)-1)
       deallocate(cev,iev)
       allocate(cev(cfrom(IE)+nev(IE)-1),iev(cfrom(IE)+nev(IE)-1))
       call eqvr(cev,tem,cfrom(IE)-1)
       call eqvi(iev,item,cfrom(IE)-1)
       deallocate(tem,item)
      endif
c     cev real*8 iev integer*4:
      write(6,*)'allocate',(12*(cfrom(IE)+nev(IE)-1))/1024,' kb'
c     write it to cev
      do 2 J=1,nev(IE)
      iev(cfrom(IE)+J-1)=i1(J)
2     cev(cfrom(IE)+J-1)=c1(J)
c     rem: cannot renormalize to previous vectors
C     Renormalize:
      call normvc(cev,cfrom(IE),cfrom(IE)+nev(IE)-1,
     1nstartt,nendt,nproc,sar)
      call normvc(YNEW,1,N,nstartt,nendt,nproc,sar)

C     Save values & vectors into scratch-files:
      REWIND 45
      DO 10031 J=1,J1-1
10031 READ(45)
      DO 1003 J=1,IE-1
1003  READ(45)
      WRITE(45)nev(IE),(cev(I),I=cfrom(IE),cfrom(IE)+nev(IE)-1),
     1(iev(I),I=cfrom(IE),cfrom(IE)+nev(IE)-1)
c     Check energy after coefficient trimming:
      DO 210 I=1,N
210   AXV(I)=0.0d0
      if(l12)then
       DO 213 I=1,N
       DO 213 ii=1,nn(I)
       J=mm(I)+ii
       if(lcoef.and.jbb(J).eq.I)then
        abbm=abb(J)-acoef*YNEW(jbb(J))**2
       else
        abbm=abb(J)
       endif
213    AXV(I)=AXV(I)+abbm*YNEW(jbb(J))
      else
       DO 211 I=1,N
       DO 211 ii=1,nn(I)
       J=jbb(mm(I)+ii)
       if(lcoef.and.J.eq.I)then
        abbm1=abb(mm(I)+ii)-acoef*YNEW(I)**2
        abbm2=abb(mm(I)+ii)-acoef*YNEW(J)**2
       else
        abbm1=abb(mm(i)+ii)
        abbm2=abb(mm(i)+ii)
       endif
       AXV(J)=AXV(J)+abbm1*YNEW(I)
211    AXV(I)=AXV(I)+abbm2*YNEW(J)
      endif
      SUM=0.0d0
      do 212 I=1,N
212   SUM=SUM+AXV(I)*YNEW(I)
      write(6,6002)SUM
      write(3,6002)SUM
6002  format(' Final energy       : ',F20.10)

      if(lcoef)then
       ecoef=0.0d0
       do 214 I=1,N
214    ecoef=ecoef-acoef*(YNEW(I)**2)**2
       D(IE)=D(IE)-ecoef
       write(6,6003)ecoef
       write(3,6003)ecoef
6003   format(' Coupling c^4 energy: ',F20.10)
      endif

      write(6,6005)D(IE)
      write(3,6005)D(IE)
6005  format(' Lambda recorded    : ',F20.10,/)


      OPEN(4,FILE=s1//'E.SCR',FORM='UNFORMATTED')
      WRITE(4)(D(IEE),IEE=1,IE)
      CLOSE(4)

      IF(HW)THEN
       WRITE(3,*)' The eigenvector:'
       WRITE(3,3004)(YNEW(J),J=1,N)
3004   FORMAT(7F10.5)
      ENDIF

      call stp1(ELMAX)

      IF(EL.GT.ELMAX.OR.IE.GE.IEMAX) GOTO 6666
5555  CONTINUE
6666  M=MSAVE
      MV=MIN(IE,NE)
      CLOSE(2)
      CLOSE(45)
      RETURN
      END

      subroutine loadh(N,j33,a33,jbb,abb,nn,mm,s1)
      real*8 a33(*),abb(*)
      integer*4 N,j33(*),jbb(*),nn(*),mm(*),I,ii,iel
      logical l12
      character*1 s1
      iel=0
      inquire(FILE=s1//'A2.SCR',exist=l12)
      if(l12)then
       OPEN(2,FILE=s1//'A2.SCR',FORM='UNFORMATTED')
      else
       OPEN(2,FILE=s1//'A.SCR',FORM='UNFORMATTED')
      endif
      read(2)
      DO 223 I=1,N
      READ(2)nn(I),(j33(ii),ii=1,nn(I)),(a33(ii),ii=1,nn(I))
      mm(i)=iel
      DO 223 ii=1,nn(I)
      iel=iel+1
      jbb(iel)=j33(ii)
223   abb(iel)=a33(ii)
      close(2)
      write(6,4900)int((dble(iel)*2.5d0+dble(N)*2.0d0)/250.0d0)
      write(3,4900)int((dble(iel)*2.5d0+dble(N)*2.0d0)/250.0d0)
c    1int((dble(iel)*8.0d0+dble(iel)*4.0d0+dble(N)*8.0d0)/1000.0d0)
4900  format(' Matrix loaded into memory',i12,' kb')
      return
      end

      subroutine normvc(c,n1,n2,nstartt,nendt,nproc,sar)
      implicit none
      real*8 c(*),ANORM,sar(*)
      integer*4 n1,n2,i,nstartt(*),nendt(*),OMP_GET_THREAD_NUM,
     1nproc,ipr
      call ppd(nproc,nproc,nstartt,nendt,n2-n1+1)
C$OMP PARALLEL PRIVATE(i,ANORM,ipr)  DEFAULT(SHARED)
      ipr=OMP_GET_THREAD_NUM()+1
      ANORM=0.0d0
      DO 1 i=nstartt(ipr)+n1-1,nendt(ipr)+n1-1
1     ANORM=ANORM+c(i)**2
      sar(ipr)=ANORM
C$OMP END PARALLEL
      ANORM=0.0d0
      do 2 i=1,nproc
2     ANORM=ANORM+sar(i)
      ANORM=1.0d0/dsqrt(ANORM)
C$OMP PARALLEL DO PRIVATE(i)  DEFAULT(SHARED)
      do 3 i=n1,n2
3     c(i)=c(i)*ANORM
      return
      end

      subroutine eqvr(tem,cev,n)
      implicit none
      integer*4 i,n
      real*8 tem(*),cev(*)
      do 1 i=1,n
1     tem(i)=cev(i)
      return
      end

      subroutine eqvi(tem,cev,n)
      implicit none
      integer*4 i,n,tem(*),cev(*)
      do 1 i=1,n
1     tem(i)=cev(i)
      return
      end

      SUBROUTINE STP1(VMM)
      IMPLICIT none
      REAL*8 VMM
      integer*4 dum,i
      character*4 OPTION
      character*15 STRING
      open(2,file='S4.OPT')
1     READ(2,1111,END=99,ERR=99)STRING
1111  FORMAT(A15)
      do 5 i=1,15
5     if(STRING(i:i).ne.' ')goto 6
6     OPTION=STRING(i:i+3)
      IF(OPTION.EQ.'MITI')THEN
       read(2,*)
       READ(2,*)dum,VMM
       write(6,60)VMM
60     format(' Energy limit ',F12.2,' cm-1')
      ENDIF
      goto 1
99    close(2)
      RETURN
      END
